#if !defined(TOOL_MSIMG32_H)
#define TOOL_MSIMG32_H

#if !defined(OWL_MODULE_H)
# include <owl/module.h>
#endif
#if !defined(WINSYS_COLOR_H)
# include <winsys/color.h>
#endif
#if !defined(_WINGDI_)
# include <win32/wingdi.h>
#endif


// Currently defined blend function
//
#define AC_SRC_OVER            0x00

// Alpha format flags
//
#define AC_SRC_ALPHA           0x01

// Gradient drawing modes
//
#define GRADIENT_FILL_RECT_H   0x00000000
#define GRADIENT_FILL_RECT_V   0x00000001
#define GRADIENT_FILL_TRIANGLE 0x00000002
#define GRADIENT_FILL_OP_FLAG  0x000000ff

typedef USHORT COLOR16;

typedef struct _TRIVERTEX
{
  LONG    x;
  LONG    y;
  COLOR16 Red;
  COLOR16 Green;
  COLOR16 Blue;
  COLOR16 Alpha;
} TRIVERTEX, *PTRIVERTEX, *LPTRIVERTEX;

typedef struct _GRADIENT_TRIANGLE
{
  ULONG Vertex1;
  ULONG Vertex2;
  ULONG Vertex3;
} GRADIENT_TRIANGLE, *PGRADIENT_TRIANGLE, *LPGRADIENT_TRIANGLE;

typedef struct _GRADIENT_RECT
{
  ULONG UpperLeft;
  ULONG LowerRight;
} GRADIENT_RECT, *PGRADIENT_RECT, *LPGRADIENT_RECT;

typedef struct _BLENDFUNCTION
{
  BYTE BlendOp;
  BYTE BlendFlags;
  BYTE SourceConstantAlpha;
  BYTE AlphaFormat;
} BLENDFUNCTION, *PBLENDFUNCTION;

//
// class TMSImage32Module
// ~~~~~ ~~~~~~~~~~~~~~~~
// TMSImage32Module is a singleton that
// encapsulates msimg32.dll
//
class TMSImage32Module : public TModule {
  public:
    static TMSImage32Module& GetInstance();

    TModuleProc11<BOOL, HDC, int, int, int, int, HDC, int, int, int, int, BLENDFUNCTION> AlphaBlend;
    TModuleProc6 <BOOL, HDC, PTRIVERTEX, ULONG, PVOID, ULONG, ULONG> GradientFill;
    TModuleProc11<BOOL, HDC, int, int, int, int, HDC, int, int, int, int, UINT> TransparentBlt;

  private:
    TMSImage32Module();
};


//
// class TMSImage32Helper
// ~~~~~ ~~~~~~~~~~~~~~~~
// This class provides helper functions for
// MSImage32 module.
//
class TMSImage32Helper {
  public:
    TMSImage32Helper(TMSImage32Module& module);

    void DrawGradientRect(HDC dc, TRect& rect, TColor color1, TColor color2,
                          ULONG mode);
    void DrawGradientRect(HDC dc, int left, int top, int right, int bottom,
                          TColor color1, TColor color2, ULONG mode);

  private:
    TMSImage32Module& Module;
};

#endif  // TOOL_MSIMG32_H
