#include <owl/applicat.h>
#include <owl/buttonga.h>
#include <owl/decframe.h>

#include "fltbtnga.h"
#include "xpctlbar.h"
#include "test.rc"

class TTestWin : public TWindow {
  public:
    TTestWin(TWindow* parent);

  protected:
    void CmFileNew      ();
    void CmFileOpen     ();
    void CmFileSave     ();
    void CmFileSaveAs   ();
    void CmFileSaveAll  ();

    void CmFilePrint    ();
    void CmFilePageSetup();

    void CmEditUndo ();
    void CmEditRedo ();

    void CmEditCut  ();
    void CmEditCopy ();
    void CmEditPaste();

    void CeEnabler(TCommandEnabler& ce);

  private:
    bool flag;

  DECLARE_RESPONSE_TABLE(TTestWin);
};

DEFINE_RESPONSE_TABLE1(TTestWin, TWindow)
  EV_COMMAND(CM_FILENEW,       CmFileNew),
  EV_COMMAND(CM_FILEOPEN,      CmFileOpen),
  EV_COMMAND(CM_FILESAVE,      CmFileSave),
  EV_COMMAND(CM_FILESAVEAS,    CmFileSaveAs),
  EV_COMMAND(CM_FILESAVEALL,   CmFileSaveAll),

  EV_COMMAND(CM_FILEPRINT,     CmFilePrint),
  EV_COMMAND(CM_FILEPAGESETUP, CmFilePageSetup),

  EV_COMMAND(CM_EDITUNDO,  CmEditUndo),
  EV_COMMAND(CM_EDITREDO,  CmEditRedo),

  EV_COMMAND(CM_EDITCUT,   CmEditCut),
  EV_COMMAND(CM_EDITCOPY,  CmEditCopy),
  EV_COMMAND(CM_EDITPASTE, CmEditPaste),

  EV_COMMAND_ENABLE(CM_FILENEW,  CeEnabler),
  EV_COMMAND_ENABLE(CM_FILEOPEN, CeEnabler),
END_RESPONSE_TABLE;

TTestWin::TTestWin(TWindow* parent)
:
 TWindow(parent),
 flag(true)
{
}

void
TTestWin::CmFileNew()
{
  flag = !flag;
}

void
TTestWin::CmFileOpen()
{
  flag = !flag;
}

void
TTestWin::CmFileSave()
{
}

void
TTestWin::CmFileSaveAs()
{
}

void
TTestWin::CmFileSaveAll()
{
}

void
TTestWin::CmFilePrint()
{
}

void
TTestWin::CmFilePageSetup()
{
}

void
TTestWin::CmEditUndo()
{
}

void
TTestWin::CmEditRedo()
{
}

void
TTestWin::CmEditCut()
{
}

void
TTestWin::CmEditCopy()
{
}

void
TTestWin::CeEnabler(TCommandEnabler& ce)
{
  switch (ce.GetId()) {
    case CM_FILENEW:
      ce.SetCheck(flag);
      break;

    case CM_FILEOPEN:
      ce.SetCheck(!flag);
      break;
  }
}

void
TTestWin::CmEditPaste()
{
  MessageBox("Paste operation is to be performed...", "Test", MB_OK);
}

class TTestApp : public TApplication {
  public:
    TTestApp(const char* name);

    void InitMainWindow();
};

TTestApp::TTestApp(const char* name)
:
 TApplication(name)
{
}

void
TTestApp::InitMainWindow()
{
  TDecoratedFrame* frame = new TDecoratedFrame(0, "Test Application",
                                               new TTestWin((TWindow*)0));
  frame->AssignMenu(IDM_MENU);

  TXpDockableControlBar* cb1 = new TXpDockableControlBar(frame);
  cb1->Insert(*new TFlatGripperGadget);
  cb1->Insert(*new TFlatButtonGadget(CM_FILENEW,       CM_FILENEW));
  cb1->Insert(*new TFlatButtonGadget(CM_FILEOPEN,      CM_FILEOPEN));
  cb1->Insert(*new TFlatButtonGadget(CM_FILESAVE,      CM_FILESAVE));
  cb1->Insert(*new TFlatButtonGadget(CM_FILESAVEALL,   CM_FILESAVEALL));
  cb1->Insert(*new TFlatSeparatorGadget);
  cb1->Insert(*new TFlatButtonGadget(CM_FILEPRINT,     CM_FILEPRINT));
  cb1->Insert(*new TFlatButtonGadget(CM_FILEPAGESETUP, CM_FILEPAGESETUP));
  cb1->SetHintMode(TGadgetWindow::EnterHints);

  TXpDockableControlBar* cb2 = new TXpDockableControlBar(frame);
  cb2->Insert(*new TFlatGripperGadget);
  cb2->Insert(*new TFlatButtonGadget(CM_EDITUNDO,  CM_EDITUNDO));
  cb2->Insert(*new TFlatButtonGadget(CM_EDITREDO,  CM_EDITREDO));
  cb2->Insert(*new TFlatSeparatorGadget);
  cb2->Insert(*new TFlatButtonGadget(CM_EDITCUT,   CM_EDITCUT));
  cb2->Insert(*new TFlatButtonGadget(CM_EDITCOPY,  CM_EDITCOPY));
  cb2->Insert(*new TFlatButtonGadget(CM_EDITPASTE, CM_EDITPASTE));
  cb2->SetHintMode(TGadgetWindow::EnterHints);

  THarbor* harbor = new THarbor(*frame);
  harbor->Insert(*cb1, alTop);
  harbor->Insert(*cb2, alTop);

  SetMainWindow(frame);
}

int
OwlMain(int /* argc */, char* /* argv */ [])
{
  return TTestApp("Test Application").Run();
}
