#include "msimg32.h"
#include "toolhlpr.h"
#include "xpctlbar.h"

// TXpControlBar implementation
//
DEFINE_RESPONSE_TABLE1(TXpControlBar, TControlBar)
  EV_WM_ERASEBKGND,
END_RESPONSE_TABLE;

TXpControlBar::TXpControlBar(TWindow* parent, TTileDirection direction,
                             TFont* font, TModule* module)
:
 TControlBar(parent, direction, font, module)
{
  // Modify margins to have a narrower control bar
  //
  Margins.Units = TMargins::BorderUnits;

  Margins.Left = Margins.Right  = 2;
  Margins.Top  = Margins.Bottom = 2;
}

void
TXpControlBar::Paint(TDC& dc, bool erase, TRect& rect)
{
  TRect targetRect;
  GetClientRect(targetRect);

  if (targetRect.left != rect.left || targetRect.right != rect.right) {
    targetRect.left  = rect.left;
    targetRect.right = rect.right;

    PaintBackground(dc.GetHDC(), targetRect);
  }

  TControlBar::Paint(dc, erase, rect);
}

bool
TXpControlBar::EvEraseBkgnd(HDC hDC)
{
  TRect rect;
  GetClientRect(rect);

  PaintBackground(hDC, rect);

  return true;
}

void
TXpControlBar::PaintBackground(HDC hDC, TRect& rect)
{
  TMSImage32Helper helper(TMSImage32Module::GetInstance());

  TColor nrBarColor = TColor::Sys3dFace;
  TColor ltBarColor = LightenColor(nrBarColor, 20);
  TColor dkBarColor = DarkenColor (nrBarColor, 40);

  helper.DrawGradientRect(hDC, rect, ltBarColor, dkBarColor,
                          GRADIENT_FILL_RECT_V);
}

// TXpDockableControlBar implementation
//
DEFINE_RESPONSE_TABLE1(TXpDockableControlBar, TDockableControlBar)
  EV_WM_ERASEBKGND,
END_RESPONSE_TABLE;

TXpDockableControlBar::TXpDockableControlBar(TWindow* parent,
                                             TTileDirection direction,
                                             TFont* font, TModule* module)
:
 TDockableControlBar(parent, direction, font, module)
{
  // Modify margins to have a narrower control bar
  //
  Margins.Units = TMargins::BorderUnits;

  Margins.Left = Margins.Right  = 2;
  Margins.Top  = Margins.Bottom = 2;
}

void
TXpDockableControlBar::Paint(TDC& dc, bool erase, TRect& rect)
{
  TRect targetRect;
  GetClientRect(targetRect);

  TTileDirection direction = GetDirection();
  if (direction == Horizontal) {
    if (targetRect.left != rect.left || targetRect.right != rect.right) {
      targetRect.left  = rect.left;
      targetRect.right = rect.right;

      PaintBackgroundHorizontally(dc.GetHDC(), targetRect);
    }
  }
  else if (direction == Vertical) {
    if (targetRect.top != rect.top || targetRect.bottom != rect.bottom) {
      targetRect.top    = rect.top;
      targetRect.bottom = rect.bottom;

      PaintBackgroundVertically(dc.GetHDC(), targetRect);
    }
  }
  else if (targetRect != rect)
    FillSolidRect(dc.GetHDC(), targetRect, TColor::Sys3dFace);

  TDockableControlBar::Paint(dc, erase, rect);
}

bool
TXpDockableControlBar::EvEraseBkgnd(HDC hDC)
{
  TRect rect;
  GetClientRect(rect);

  FillSolidRect(hDC, rect, TColor::Sys3dFace);

  TTileDirection direction = GetDirection();
  if (direction == Horizontal) {
    rect.left++;

    PaintBackgroundHorizontally(hDC, rect);
  }
  else if (direction == Vertical) {
    rect.top++;

    PaintBackgroundVertically(hDC, rect);
  }

  if (direction != Rectangular) {
    rect.right  -= 1;
    rect.bottom -= 1;

    PaintCorners(hDC, rect, TColor::Sys3dLight);
  }

  return true;
}

void
TXpDockableControlBar::PaintBackgroundHorizontally(HDC hDC, TRect& rect)
{
  TMSImage32Helper helper(TMSImage32Module::GetInstance());

  TColor nrBarColor = TColor::Sys3dFace;
  TColor ltBarColor = LightenColor(nrBarColor, 20);
  TColor dkBarColor = DarkenColor (nrBarColor, 40);

  helper.DrawGradientRect(hDC, rect, ltBarColor, dkBarColor,
                          GRADIENT_FILL_RECT_V);
}

void
TXpDockableControlBar::PaintBackgroundVertically(HDC hDC, TRect& rect)
{
  TMSImage32Helper helper(TMSImage32Module::GetInstance());

  TColor nrBarColor = TColor::Sys3dFace;
  TColor ltBarColor = LightenColor(nrBarColor, 20);
  TColor dkBarColor = DarkenColor (nrBarColor, 40);

  helper.DrawGradientRect(hDC, rect, ltBarColor, dkBarColor,
                          GRADIENT_FILL_RECT_H);
}

void
TXpDockableControlBar::PaintCorners(HDC hDC, TRect& rect, TColor color)
{
  TDC dc(hDC);

  dc.SetPixel(rect.left,      rect.top,        color);
  dc.SetPixel(rect.left + 1,  rect.top,        color);
  dc.SetPixel(rect.left + 2,  rect.top,        color);
  dc.SetPixel(rect.left,      rect.top + 1,    color);
  dc.SetPixel(rect.left,      rect.top + 2,    color);
  dc.SetPixel(rect.left + 1,  rect.top + 1,    color);

  dc.SetPixel(rect.left,      rect.bottom,     color);
  dc.SetPixel(rect.left + 1,  rect.bottom,     color);
  dc.SetPixel(rect.left + 2,  rect.bottom,     color);
  dc.SetPixel(rect.left,      rect.bottom - 1, color);
  dc.SetPixel(rect.left,      rect.bottom - 2, color);
  dc.SetPixel(rect.left + 1,  rect.bottom - 1, color);

  dc.SetPixel(rect.right,     rect.top,        color);
  dc.SetPixel(rect.right - 1, rect.top,        color);
  dc.SetPixel(rect.right - 2, rect.top,        color);
  dc.SetPixel(rect.right,     rect.top + 1,    color);
  dc.SetPixel(rect.right,     rect.top + 2,    color);
  dc.SetPixel(rect.right - 1, rect.top + 1,    color);

  dc.SetPixel(rect.right,     rect.bottom,     color);
  dc.SetPixel(rect.right - 1, rect.bottom,     color);
  dc.SetPixel(rect.right - 2, rect.bottom,     color);
  dc.SetPixel(rect.right,     rect.bottom - 1, color);
  dc.SetPixel(rect.right,     rect.bottom - 2, color);
  dc.SetPixel(rect.right - 1, rect.bottom - 1, color);
}

